/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */

#include "config.h"

#include <wtf/GetPtr.h>
#include "CSSRule.h"
#include "CSSValue.h"
#include "CanvasPixelArray.h"
#include "DOMWindow.h"
#include "Database.h"
#include "Element.h"
#include "Event.h"
#include "GDOMBinding.h"
#include "GDOMObject.h"
#include "HTMLCollection.h"
#include "Node.h"
#include "StyleSheet.h"
#include "Text.h"
#include "gdom/GStringConvert.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomDatabase.h"
#include "gdom/PrivateGdomDOMObject.h"
#include "gdom/PrivateGdomDatabase.h"
#include "gdom/PrivateIntGdomDatabase.h"

namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore

namespace WebCore {
gpointer toGDOM(WebCore::Database* obj)
{
    return WebCore::cacheDOMObject<WebCore::Database, GDOMDatabase>(obj);
}
} /* namespace WebCore */
extern "C" {

GDOMDatabase::GDOMDatabase(PassRefPtr<WebCore::Database> impl)
    : GDOMObject()
    , m_impl(impl)
{
}

GDOMDatabase::~GDOMDatabase()
{
    m_impl.get()->deref();
    WebCore::GDOMScriptInterpreter::forgetDOMObject(m_impl.get());

}


/* XXX TODO: custom function gdom_database_change_version */


/* XXX TODO: custom function gdom_database_transaction */

} /* extern "C" */


#include "config.h"
#include "glib-object.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitprivate.h> 
#include "DOMWindow.h" /* yukk: TODO - make this optional */
#include "Node.h" /* yukk: TODO - make this optional */
#include "NamedNodeMap.h" /* yukk: TODO - make this optional */
#include "CanvasPattern.h" /* yukk: TODO - make this optional */
#include "CanvasGradient.h" /* yukk: TODO - make this optional */
#include "CanvasStyle.h" /* yukk: TODO - make this optional */
#include "CanvasRenderingContext2D.h" /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomDOMObject.h> /* yukk: TODO - make this optional */
#include <gdom/GdomEventTargetNode.h> /* yukk: TODO - make this optional */
#include <gdom/PrivateGdomEventTargetNode.h> /* yukk: TODO - make this optional on EventTarget*/
#include "Database.h"
#include <gdom/GdomDatabase.h>
#include <gdom/PrivateGdomDatabase.h>
#include <gdom/PrivateIntGdomDatabase.h>
#include <gdom/GdomDatabaseprops.h>

#include "gdom/GStringConvert.h"

#include "CanvasPixelArray.h"
#include "Event.h"
#include "Text.h"
#include "CSSValue.h"
#include "CSSRule.h"
#include "HTMLCollection.h"
#include "StyleSheet.h"
#include "Document.h"
#include "webkitmarshal.h"
#include <gdom/GdomEvent.h>



namespace WebCore {
    gpointer toGDOM(Node* node);
    gpointer toGDOM(CSSValue* node);
    gpointer toGDOM(CSSRule* node);
    gpointer toGDOM(Element* node);
    gpointer toGDOM(Document* node);
    gpointer toGDOM(HTMLCollection* node);
    //gpointer toGDOM(SVGPathSeg* node);
    gpointer toGDOM(StyleSheet* node);
    gpointer toGDOM(EventTargetNode* node);
    gpointer toGDOM(EventTarget* node);
    gpointer toGDOM(Event* node);
    gpointer toGDOM(CanvasPixelArray* node);
    gpointer toGDOM(DOMWindow* node);
    gpointer toGDOM(Text* node);
} // namespace WebCore



G_BEGIN_DECLS

G_DEFINE_TYPE(GdomDatabase, gdom_database, GDOM_TYPE_DOM_OBJECT);

/*
struct _GdomDatabasePrivate {
    class WebCore::Database *item;
};

#define GDOM_DATABASE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), GDOM_TYPE_DATABASE, GdomDatabasePrivate))
*/
G_END_DECLS
WebCore::Database *
gdom_database_get_private(GdomDatabase *request)
{
    GDOMDatabase *res;
    res = static_cast<GDOMDatabase*>(gdom_dom_object_get_private(GDOM_DOM_OBJECT(request)));
    return static_cast<WebCore::Database*>(res->impl());
}

G_BEGIN_DECLS

enum {
    BROWSER_EVENT,
    LAST_SIGNAL
};

static guint event_signals[LAST_SIGNAL] = { 0, };

enum {
    PROP_0,
    PROP_VERSION,
};


static void gdom_database_finalize(GObject* object)
{
    /*GdomDatabase* request = GDOM_DATABASE(object);*/
    /*GdomDatabasePrivate* priv = request->priv;*/

    /*GObjectClass *parent_class = g_type_class_peek (g_type_peek_parent (GDOM_TYPE_DOM_OBJECT));*/

    /* Free any memory or close any files */

    /*parent_class-> finalize (object);*/

    G_OBJECT_CLASS(gdom_database_parent_class)->finalize(object);
}

static void gdom_database__set_property_fn(GObject * object, guint prop_id,
                     const GValue * value,
                     GParamSpec * pspec)
{
    GdomDatabase *self = GDOM_DATABASE(object);
    WebCore::Database * gself = (gdom_database_get_private(self));
    switch (prop_id) {
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_database__get_property_fn(GObject * object, guint prop_id,
                     GValue * value,
                     GParamSpec * pspec)
{
    GdomDatabase *self = GDOM_DATABASE(object);
    WebCore::Database * gself = (gdom_database_get_private(self));
    switch (prop_id) {
    case PROP_VERSION:
    {
         g_value_set_string (value, _G_GET_UTF8(gself->version()));
         break;
    }
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
        break;
    }
}


static void gdom_database_class_init(GdomDatabaseClass* requestClass)
{
    GObjectClass *g_klass = G_OBJECT_CLASS (requestClass);
    g_klass->finalize = gdom_database_finalize;
    g_klass->set_property = gdom_database__set_property_fn;
    g_klass->get_property = gdom_database__get_property_fn;

    g_object_class_install_property (g_klass, PROP_VERSION,
            g_param_spec_string ("version", /* name */
                "database_version", /* short description */
                "read-only  gchar * Database.version", /* longer - could do with some extra doc stuff here */
                 "", /* default */
                 (GParamFlags)G_PARAM_READABLE));





    /*g_type_class_remove_private(requestClass, sizeof(GdomDatabasePrivate));*/
}

static void gdom_database_init(GdomDatabase* request)
{
    /*GdomDatabasePrivate* priv = GDOM_DATABASE_GET_PRIVATE(request);
    request->priv = priv;*/
}

} /* extern "C" */

